#!/usr/bin/perl

use warnings;
use strict;

use POSIX;
use Fuse;
use Data::Dumper;

my $filesystem = {
	root => {
		content => {
			wurst => {
				type => 'dir',
				content => {
					bratwurst => {
						type => 'dir'
					},
					bockwurst => {
						type => 'dir',
						content => {
							gebraten => {
								type => 'file',
								content => "Gebratenes ist auch lecker\n"
							},
							geschmort => {
								type => 'dir'
							}
						}
					},
					teewurst => {
						type => 'dir',
						content => {
							liesmich => {
								type => 'file',
								content => "Hallo. Teewurst ist lecker!\n"
							}
						}
					}
				}
			},
			obst => {
				type => 'dir',
				content => {
					apfel => {
						type => 'dir',
						content => {
							wurm => {
								type => 'file',
								content => "Hier ist der Wurm drin.\n"
							}
						}
					}
				}
			},
			info => {
				type => 'file',
				content => "Wurst- und Obst-Filesystem V0.1\n"
			}
		}
	}
};

my $lastAttredFile;

sub my_getattr {
	my ( $filename ) = @_;
	print "Requesting Attribs [$filename]: ";

	# regulaere Datei
	my $type = 0100;

	# owner: r/w, group: r, others: r
	my $bits = 0644;

	# falls Verzeichnis, type auf dir setzen,
	# mode auf 0755:
	# owner: r/w/x, group: r/x, others: r/x
	my $current = $filesystem->{'root'}->{'content'};
	my @pathElements = split( '/', $filename );
	my $currentType = 'file';

	if ( @pathElements > 1 ) {
		foreach my $pathElement ( @pathElements[1..$#pathElements] ) {
			#print 'Checking for: ', $current->{$pathElement}, "\n";
			if ( !defined( $current->{$pathElement} ) ) {
				return -1*ENOENT;
			}
			$currentType = $current->{$pathElement}->{'type'};
			$lastAttredFile = $current->{$pathElement} if ( $currentType eq 'file' );

			$current = $current->{$pathElement}->{'content'} if ( $currentType eq 'dir' );
		}
	}

	if ( $filename eq '/' || $currentType eq 'dir' ) {
		$type = 0040;
		$bits = 0755;
		print "DIR\n"
	} else {
		print "FILE\n";
	}

	#my $mode = $type << 9 | $bits;
	my $mode = $type << 9 | $bits;
	my $nlink = 1;

	# reale UID (siehe perlvar)
	my $uid = $<;

	# reale GID (siehe perlvar)
	my ($gid) = split / /, $(;

	# Geraete-ID (special files only)
	my $rdev = 0;

	# letzter Zugriff
	my $atime = time;

	# Groeße
	my $size = 0;

	if ( $currentType eq 'file' ) {
		$size = length( $lastAttredFile->{'content'} );
	}

	# letzte Aenderung
	my $mtime = $atime;

	# letzte Aenderung Inode
	my $ctime = $atime;

	my $blksize = 1024;
	my $blocks = 1;
	
	my $dev = 0;
	my $ino = 0;

	return ( $dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize,	$blocks	);
}

sub my_getdir {
	my ( $filename ) = @_;
	print "getdir requesting [$filename]\n";

	my $current = $filesystem->{'root'}->{'content'};
	my @pathElements = split( '/', $filename );

	if ( @pathElements > 1 ) {
		foreach my $pathElement ( @pathElements[1..$#pathElements] ) {
			return( -1*ENOENT ) if ( !defined( $current->{$pathElement} ) );
			$current = $current->{$pathElement}->{'content'};
		}
	}

	return( '.', keys( %{$current} ), 0 );
}

sub my_read {
	my ( $filename ) = @_;
	return $lastAttredFile->{'content'};
}

Fuse::main(
	mountpoint	=> '/home/sxw/tmp/mnt',
	getdir		=> \&my_getdir,
	getattr		=> \&my_getattr,
	read		=> \&my_read,
	debug => 0
);

